%% load data

clc; clear all; close all;

[d,h] = load_sto('./simple_motion_states.sto');
[d4,h4] = xlsread('./Runner18_initstates.xlsx');
[d3,h3] = load_sto('./simple_motion_Actuation_force.sto');
[d5,h5] = load_sto('./simple_motion_ForceReporter_forces.sto');
[d6,h6] = load_sto('./simple_motion_pErr.sto');
[d7,h7] = load_sto('./inverse_dynamics.sto');
[d8,h8] = load_sto('Z:\Research_Projects\Schmitz\KneeModel\Detailed_Knee_Model\Passive_Flexion\inverse_dynamics.sto');
[d9,h9] = load_mot('Z:\Research_Projects\Schmitz\KneeModel\Detailed_Knee_Model\Passive_Flexion\flexion_ang.mot');
[d10,h10] = load_sto('./simple_motion_MuscleAnalysis_Moment_knee_angle_r.sto');
[d11,h11] = load_sto('./simple_motion_MuscleAnalysis_PassiveFiberForce.sto');
[d12,h12] = load_sto('./simple_motion_MuscleAnalysis_NormalizedFiberLength.sto');
% [d13,h13] = load_sto('./Runner18_StaticOptimization_force.sto');
[d14,h14] = load_sto('./simple_motion_controls.sto');
[d15,h15] = load_sto('./simple_motion_MuscleAnalysis_ActiveFiberForce.sto');
t_cut = 0.23;

%% plot kinematics

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

t1 = d(:,strcmp(h,'time'));
t4 = d4(:,strcmp(h4,'time'));
ii1 = find(t1>t_cut);
ii4 = find(t4>t_cut);

subplot(3,2,1);
plot(t1(ii1),(180/3.14)*d(ii1,strcmp(h,'knee_angle_r')),'b-','LineWidth',3);
hold on;
plot(t4(ii4),(180/3.14)*d4(ii4,strcmp(h4,'knee_angle_r')),'r:','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Knee Flexion (deg)');
legend('Tracked Angle','Prescribed');
xlim([t_cut 1]);
set(gca,'XTickLabel',{' '});

subplot(3,2,2);
plot(t1(ii1),d(ii1,strcmp(h,'knee_tx_r')),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Anterior Translation (m)');
xlim([t_cut 1]);
set(gca,'XTickLabel',{' '});

subplot(3,2,3);
plot(t1(ii1),(180/3.14)*d(ii1,strcmp(h,'knee_rot_r')),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Knee Internal Rotation (deg)');
xlim([t_cut 1]);
set(gca,'XTickLabel',{' '});

subplot(3,2,4);
plot(t1(ii1),d(ii1,strcmp(h,'knee_ty_r')),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Superior Translation (m)');
xlim([t_cut 1]);
set(gca,'XTickLabel',{' '});

subplot(3,2,5);
plot(t1(ii1),(180/3.14)*d(ii1,strcmp(h,'knee_add_r')),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
xlabel('Time (s)');
ylabel('Knee Adduction (deg)');

subplot(3,2,6);
plot(t1(ii1),d(ii1,strcmp(h,'knee_tz_r')),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
xlabel('Time (s)');
ylabel('Lateral Translation (m)');

saveas(gcf,'simple_motion5_kinematics.fig');
saveas(gcf,'simple_motion5_kinematics.jpg');

%% plot forces

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

t3 = d3(:,strcmp(h3,'time'));
ii3 = find(t3>t_cut);

t5 = d5(:,strcmp(h5,'time'));
ii5 = find(t5>t_cut);

% knee torque reserve actuator
subplot(2,3,2);
plot(t3(ii3),(d3(ii3,strcmp(h3,'knee_angle_reserve'))),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
xlabel('Time (s)');
ylabel('Torque (Nm)');
title('Knee Angle Reserve Actuator');

% muscle forces
subplot(2,3,4);
plot(t3(ii3),d3(ii3,strcmp(h3,'bifemlh_r')),'b-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'bifemsh_r')),'r-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'grac_r')),'g-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'lat_gas_r')),'k-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'med_gas_r')),'b:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'sar_r')),'r:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'semimem_r')),'g:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'semiten_r')),'k:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'rect_fem_r')),'b-','LineWidth',1);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'vas_int_r')),'r-','LineWidth',1);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'vas_lat_r')),'g-','LineWidth',1);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'vas_med_r')),'k-','LineWidth',1);
set(gca,'box','off','FontSize',12);
xlabel('Time (s)');
ylabel('Force (N)');
title('Muscle Actuators');
legend('Biceps Femoris Long Head','Biceps Femoris Short Head','Gracilis','Lateral Gastrocnemius',...
    'Medial Gastrocnemius','Sartorius','Semimembranosus','Semitendinosus','Rectus Femoris',...
    'Vastus Intermedius','Vastus Lateralis','Vastus Medialis');

% ligament forces
subplot(2,3,6);
plot(t5(ii5),d5(ii5,strcmp(h5,'aACL')),'b-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pACL')),'r-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'aPCL')),'g-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pPCL')),'k-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'LCL')),'b:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'PFL')),'r:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPa')),'g:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPl')),'k:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPo')),'b-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPm')),'r-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'aMCL')),'g-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'iMCL')),'k-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pMCL')),'m-','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'aCM')),'m:','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pCM')),'m-.','LineWidth',1);
set(gca,'box','off','FontSize',12);
xlabel('Time (s)');
ylabel('Force (N)');
title('Ligament');
legend('aACL','pACL','aPCL','pPCL','LCL','PFL','CAPa','CAPl','CAPo','CAPm','aMCL','iMCL','pMCL','aCM','pCM');

% contact forces

mFX = d5(ii5,strcmp('medial_tf.femur_r.force.X',h5));
mFY = d5(ii5,strcmp('medial_tf.femur_r.force.Y',h5));
mFZ = d5(ii5,strcmp('medial_tf.femur_r.force.Z',h5));
mTX = d5(ii5,strcmp('medial_tf.femur_r.torque.X',h5));
mTY = d5(ii5,strcmp('medial_tf.femur_r.torque.Y',h5));
mTZ = d5(ii5,strcmp('medial_tf.femur_r.torque.Z',h5));

lFX = d5(ii5,strcmp('lateral_tf.femur_r.force.X',h5));
lFY = d5(ii5,strcmp('lateral_tf.femur_r.force.Y',h5));
lFZ = d5(ii5,strcmp('lateral_tf.femur_r.force.Z',h5));
lTX = d5(ii5,strcmp('lateral_tf.femur_r.torque.X',h5));
lTY = d5(ii5,strcmp('lateral_tf.femur_r.torque.Y',h5));
lTZ = d5(ii5,strcmp('lateral_tf.femur_r.torque.Z',h5));

[med_mag_force] = calc_magnitude(mFX,mFY,mFZ);
[lat_mag_force] = calc_magnitude(lFX,lFY,lFZ);

subplot(2,3,5);
plot(t5(ii5),lat_mag_force,'b-','LineWidth',3);
hold on;
plot(t5(ii5),med_mag_force,'r-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Force (N)');
title('Contact Force');
legend('Lateral','Medial');
xlabel('Time (s)');

saveas(gcf,'simple_motion5_forces.fig');
saveas(gcf,'simple_motion5_forces.jpg');

%% knee angle, ID moment, reserve moment

t6 = d6(:,strcmp(h6,'time'));
ii6 = find(t6>t_cut);

t7 = d7(:,strcmp(h7,'time'));
ii7 = find(t7>t_cut);

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

subplot(3,1,1)
plot(t1(ii1),(180/3.14)*d(ii1,strcmp(h,'knee_angle_r')),'b-','LineWidth',3);
hold on;
plot(t4(ii4),(180/3.14)*d4(ii4,strcmp(h4,'knee_angle_r')),'r:','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Knee Flexion (deg)');
legend('Tracked Angle','Prescribed');
xlim([t_cut 1]);
set(gca,'XTickLabel',{' '});

subplot(3,1,2);
plot(t6(ii6),(180/3.14)*abs(d6(ii6,strcmp(h6,'knee_angle_r'))),'r:','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Knee Flexion Error (deg)');
xlim([t_cut 1]);
xlabel('Time (s)');

fcut = 10;

subplot(3,1,3)

tt = t7(ii7);
ktorq = (d7(ii7,strcmp(h7,'knee_angle_r_moment')));
tnew = min(tt):0.001:max(tt);
fs = 1/0.001;
ktorq_new = interp1(tt,ktorq,tnew);
[b,a] = butter(4,fcut/fs,'low');
ktorq_new_filt = filtfilt(b,a,ktorq_new);
plot(tnew,ktorq_new_filt,'r:','LineWidth',3);
hold on;

tt3 = t3(ii3);
ktorqr = (d3(ii3,strcmp(h3,'knee_angle_reserve')));
tnew = min(tt3):0.001:max(tt3);
ktorqr_new = interp1(tt3,ktorqr,tnew);
[b,a] = butter(4,fcut/fs,'low');
ktorqr_new_filt = filtfilt(b,a,ktorqr_new);
plot(tnew,ktorqr_new_filt,'b-','LineWidth',3);
 
% [d8,h8] = load_sto('Z:\Research_Projects\Schmitz\KneeModel\Detailed_Knee_Model\Passive_Flexion\inverse_dynamics.sto');
% t7 = d7(:,strcmp(h7,'time'));
% ii7 = find(t7>t_cut);

set(gca,'box','off','FontSize',12);
xlabel('Time (s)');
ylabel('Torque (Nm)');
legend('Inverse Dynamics Torque','Knee Angle Reserve Actuator');

saveas(gcf,'simple_motion5_F2.fig');
saveas(gcf,'simple_motion5_F2.jpg');

%% muscle, contact, ligament forces

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

% muscle forces
subplot(3,1,1);
plot(t3(ii3),d3(ii3,strcmp(h3,'bifemlh_r')),'b-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'bifemsh_r')),'r-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'grac_r')),'g-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'lat_gas_r')),'k-','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'med_gas_r')),'b:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'sar_r')),'r:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'semimem_r')),'g:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'semiten_r')),'k:','LineWidth',3);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'rect_fem_r')),'b-','LineWidth',1);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'vas_int_r')),'r-','LineWidth',1);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'vas_lat_r')),'g-','LineWidth',1);
hold on;
plot(t3(ii3),d3(ii3,strcmp(h3,'vas_med_r')),'k-','LineWidth',1);
set(gca,'box','off','FontSize',12);
set(gca,'XTickLabel',{' '});
ylabel('Force (N)');
title('Muscle Actuators');
legend('Biceps Femoris Long Head','Biceps Femoris Short Head','Gracilis','Lateral Gastrocnemius',...
    'Medial Gastrocnemius','Sartorius','Semimembranosus','Semitendinosus','Rectus Femoris',...
    'Vastus Intermedius','Vastus Lateralis','Vastus Medialis');

% ligament forces
subplot(3,1,2);
plot(t5(ii5),d5(ii5,strcmp(h5,'aACL')),'b-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pACL')),'r-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'aPCL')),'g-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pPCL')),'k-','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'LCL')),'b:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'PFL')),'r:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPa')),'g:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPl')),'k:','LineWidth',3);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPo')),'b-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'CAPm')),'r-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'aMCL')),'g-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'iMCL')),'k-.','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pMCL')),'m-','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'aCM')),'m:','LineWidth',1);
hold on;
plot(t5(ii5),d5(ii5,strcmp(h5,'pCM')),'m-.','LineWidth',1);
set(gca,'box','off','FontSize',12);
ylabel('Force (N)');
title('Ligament');
legend('aACL','pACL','aPCL','pPCL','LCL','PFL','CAPa','CAPl','CAPo','CAPm','aMCL','iMCL','pMCL','aCM','pCM');
set(gca,'XTickLabel',{' '});

% contact forces

mFX = d5(ii5,strcmp('medial_tf.femur_r.force.X',h5));
mFY = d5(ii5,strcmp('medial_tf.femur_r.force.Y',h5));
mFZ = d5(ii5,strcmp('medial_tf.femur_r.force.Z',h5));
mTX = d5(ii5,strcmp('medial_tf.femur_r.torque.X',h5));
mTY = d5(ii5,strcmp('medial_tf.femur_r.torque.Y',h5));
mTZ = d5(ii5,strcmp('medial_tf.femur_r.torque.Z',h5));

lFX = d5(ii5,strcmp('lateral_tf.femur_r.force.X',h5));
lFY = d5(ii5,strcmp('lateral_tf.femur_r.force.Y',h5));
lFZ = d5(ii5,strcmp('lateral_tf.femur_r.force.Z',h5));
lTX = d5(ii5,strcmp('lateral_tf.femur_r.torque.X',h5));
lTY = d5(ii5,strcmp('lateral_tf.femur_r.torque.Y',h5));
lTZ = d5(ii5,strcmp('lateral_tf.femur_r.torque.Z',h5));

[med_mag_force] = calc_magnitude(mFX,mFY,mFZ);
[lat_mag_force] = calc_magnitude(lFX,lFY,lFZ);
W = 77.5*9.81;

subplot(3,1,3);
plot(t5(ii5),lat_mag_force/W,'b-','LineWidth',3);
hold on;
plot(t5(ii5),med_mag_force/W,'r-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Force (N/weight)');
title('Contact Force');
legend('Lateral','Medial');
xlabel('Time (s)');

saveas(gcf,'simple_motion5_F3.fig');
saveas(gcf,'simple_motion5_F3.jpg');

%% knee torques

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

fcut = 10;
tt4 = t4(ii4);
k_ang = -(180/3.14)*d4(ii4,strcmp(h4,'knee_angle_r'));

tt = t7(ii7);
ktorq = (d7(ii7,strcmp(h7,'knee_angle_r_moment')));
tnew = min(tt):0.001:max(tt);
fs = 1/0.001;
ktorq_new = interp1(tt,ktorq,tnew);
[b,a] = butter(4,fcut/fs,'low');
ktorq_new_filt = filtfilt(b,a,ktorq_new);
ktorq_new_filt_2 = interp1(tnew,ktorq_new_filt,tt4);
plot(k_ang,ktorq_new_filt_2,'r:','LineWidth',3);
hold on;

tt3 = t3(ii3);
ktorqr = (d3(ii3,strcmp(h3,'knee_angle_reserve')));
tnew = min(tt3):0.001:max(tt3);
ktorqr_new = interp1(tt3,ktorqr,tnew);
[b,a] = butter(4,fcut/fs,'low');
ktorqr_new_filt = filtfilt(b,a,ktorqr_new);
ktorqr_new_filt_2 = interp1(tnew,ktorqr_new_filt,tt4);
plot(k_ang,ktorqr_new_filt_2,'b-','LineWidth',3);
 
t8 = d8(:,strcmp(h8,'time'));
ii8 = find(t8>2 & t8<20);
tt8 = t8(ii8);
k_torq_passive = d8(ii8,strcmp(h8,'knee_angle_r_moment'));

t9 = d9(:,strcmp(h9,'time'));
ii9 = find(t9>2 & t9<20);
tt9 = t9(ii9);
k_ang_pass = d9(ii9,strcmp(h9,'knee_angle_r'));

k_torq_passive_2 = interp1(tt8,k_torq_passive,tt9);
hold on;
% plot(-k_ang_pass,abs(k_torq_passive_2),'g:');

set(gca,'box','off','FontSize',12);
xlabel('Knee Angle (deg)');
ylabel('Torque (Nm)');
legend('Inverse Dynamics Torque','Knee Angle Reserve Actuator');

saveas(gcf,'simple_motion5_torques.fig');
saveas(gcf,'simple_motion5_torques.jpg');

%% torque due to muscles

m_names = {'bifemlh_r';
    'bifemsh_r';
    'grac_r';
    'lat_gas_r';
    'med_gas_r';
    'sar_r';
    'semimem_r';
    'semiten_r';
    'rect_fem_r';
    'vas_int_r';
    'vas_lat_r';
    'vas_med_r'};

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

l_type = {'r-','k-','g-','b-',...
          'r:','k:','g:','b:',...
          'r.','k.','g.','b.'};

t14 = d14(:,strcmp(h14,'time'));
ii14 = find(t14>t_cut);
tt14 = t14(ii14);

t15 = d15(:,strcmp(h15,'time'));
ii15 = find(t15>t_cut);
tt15 = t14(ii15);

t10 = d10(:,strcmp(h10,'time'));
ii10 = find(t10>t_cut);
tt10 = t10(ii10);

tt = t7(ii7);
ktorq = (d7(ii7,strcmp(h7,'knee_angle_r_moment')));
tnew = min(tt):0.001:max(tt);
fs = 1/0.001;
ktorq_new = interp1(tt,ktorq,tnew);
[b,a] = butter(4,fcut/fs,'low');
ktorq_new_filt = filtfilt(b,a,ktorq_new);
plot(tnew,ktorq_new_filt,'r:','LineWidth',3);
hold on;

tt3 = t3(ii3);
ktorqr = (d3(ii3,strcmp(h3,'knee_angle_reserve')));
tnew = min(tt3):0.001:max(tt3);
ktorqr_new = interp1(tt3,ktorqr,tnew);
[b,a] = butter(4,fcut/fs,'low');
ktorqr_new_filt = filtfilt(b,a,ktorqr_new);
plot(tnew,ktorqr_new_filt,'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);

m_torq = d10(ii10,2:end);
m_torq_kang = sum(m_torq,2);
m_torq_kang_new = interp1(tt10,m_torq_kang,tnew);

hold on;
plot(tnew,m_torq_kang_new,'k-.','LineWidth',3);

hold on;
plot(tnew,m_torq_kang_new+ktorqr_new_filt,'g-.','LineWidth',3);
xlabel('Time (s)');
ylabel('Torque (Nm)');

legend('Inverse Dynamics','CMC Reserve Actuator','CMC Total Muscle Torque','CMC Muscle + CMC Reserve Actuator Torque');

saveas(gcf,'muscles2.fig');
