%% load data

clc; clear all; close all;

[d5,h5] = load_sto('./simple_motion_ForceReporter_forces.sto');
[d5s,h5s] = load_sto('../Serial/Runner18_ForceReporter_forces.sto');

%% get times

t_cut = 0.23;

t5 = d5(:,strcmp(h5,'time'));
ii5 = find(t5>t_cut);
tt5 = t5(ii5);

t5s = d5s(:,strcmp(h5s,'time'));
ii5s = find(t5s>t_cut);
tt5s = t5s(ii5s);

%% plot forces

f_cut = 10;

figure('color','w','units','normalized','outerposition',[0 0 1 1]);

% contact forces

mFX = d5(ii5,strcmp('medial_tf.femur_r.force.X',h5));
mFY = d5(ii5,strcmp('medial_tf.femur_r.force.Y',h5));
mFZ = d5(ii5,strcmp('medial_tf.femur_r.force.Z',h5));
mTX = d5(ii5,strcmp('medial_tf.femur_r.torque.X',h5));
mTY = d5(ii5,strcmp('medial_tf.femur_r.torque.Y',h5));
mTZ = d5(ii5,strcmp('medial_tf.femur_r.torque.Z',h5));

lFX = d5(ii5,strcmp('lateral_tf.femur_r.force.X',h5));
lFY = d5(ii5,strcmp('lateral_tf.femur_r.force.Y',h5));
lFZ = d5(ii5,strcmp('lateral_tf.femur_r.force.Z',h5));
lTX = d5(ii5,strcmp('lateral_tf.femur_r.torque.X',h5));
lTY = d5(ii5,strcmp('lateral_tf.femur_r.torque.Y',h5));
lTZ = d5(ii5,strcmp('lateral_tf.femur_r.torque.Z',h5));

[med_mag_force] = calc_magnitude(mFX,mFY,mFZ);
[lat_mag_force] = calc_magnitude(lFX,lFY,lFZ);
tnew = min(tt5):0.001:max(tt5);
mf = interp1(tt5,med_mag_force,tnew);
lf = interp1(tt5,lat_mag_force,tnew);
fs = 1/0.001;
[b,a] = butter(4,f_cut/(0.5*fs));
mf_filt = filtfilt(b,a,mf);
lf_filt = filtfilt(b,a,lf);

mFXs = d5s(ii5s,strcmp('medial_tf.femur_r.force.X',h5s));
mFYs = d5s(ii5s,strcmp('medial_tf.femur_r.force.Y',h5s));
mFZs = d5s(ii5s,strcmp('medial_tf.femur_r.force.Z',h5s));
mTXs = d5s(ii5s,strcmp('medial_tf.femur_r.torque.X',h5s));
mTYs = d5s(ii5s,strcmp('medial_tf.femur_r.torque.Y',h5s));
mTZs = d5s(ii5s,strcmp('medial_tf.femur_r.torque.Z',h5s));

lFXs = d5s(ii5s,strcmp('lateral_tf.femur_r.force.X',h5s));
lFYs = d5s(ii5s,strcmp('lateral_tf.femur_r.force.Y',h5s));
lFZs = d5s(ii5s,strcmp('lateral_tf.femur_r.force.Z',h5s));
lTXs = d5s(ii5s,strcmp('lateral_tf.femur_r.torque.X',h5s));
lTYs = d5s(ii5s,strcmp('lateral_tf.femur_r.torque.Y',h5s));
lTZs = d5s(ii5s,strcmp('lateral_tf.femur_r.torque.Z',h5s));

[med_mag_forces] = calc_magnitude(mFXs,mFYs,mFZs);
[lat_mag_forces] = calc_magnitude(lFXs,lFYs,lFZs);
tnews = min(tt5s):0.001:max(tt5s);
mfs = interp1(tt5s,med_mag_forces,tnews);
lfs = interp1(tt5s,lat_mag_forces,tnews);
mfs_filt = filtfilt(b,a,mfs);
lfs_filt = filtfilt(b,a,lfs);

subplot(2,3,4)
plot(tnew,mf_filt,'r:','LineWidth',3);
hold on;
plot(tnews,mfs_filt,'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Force (N)');
title('Medial Contact Force');
ylim([0 4500]);
xlabel('Time (s)');

subplot(2,3,5)
plot(tnew,lf_filt,'r:','LineWidth',3);
hold on;
plot(tnews,lfs_filt,'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
title('Lateral Contact Force');
ylim([0 4500]);
set(gca,'YTickLabel',{' '});
xlabel('Time (s)');

subplot(2,3,6)
plot(tnew,lf_filt+mf_filt,'r:','LineWidth',3);
hold on;
plot(tnews,lfs_filt+mfs_filt,'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
title('Total Contact Force');
ylim([0 4500]);
set(gca,'YTickLabel',{' '});
xlabel('Time (s)');

% muscles

knee_flex = {'bifemlh_r';
    'bifemsh_r';
    'grac_r';
    'lat_gas_r';
    'med_gas_r';
    'sar_r';
    'semimem_r';
    'semiten_r'};

ff_c = [];
ff_s = [];
for i = 1:size(knee_flex,1);
    ff_c(:,i) = d5(ii5,strcmp(h5,knee_flex{i}));
    ff_s(:,i) = d5s(ii5s,strcmp(h5s,knee_flex{i}));
end

subplot(2,3,1)
plot(tt5,sum(ff_c,2),'r:','LineWidth',3);
hold on;
plot(tt5s,sum(ff_s,2),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
ylabel('Force (N)');
title('Total Flexor Muscle Force');
set(gca,'XTickLabel',{' '});
ylim([0 4500]);

knee_ext = {'rect_fem_r';
    'vas_int_r';
    'vas_lat_r';
    'vas_med_r'};

fe_c = [];
fe_s = [];
for i = 1:size(knee_ext,1);
    fe_c(:,i) = d5(ii5,strcmp(h5,knee_ext{i}));
    fe_s(:,i) = d5s(ii5s,strcmp(h5s,knee_ext{i}));
end

subplot(2,3,2)
plot(tt5,sum(fe_c,2),'r:','LineWidth',3);
hold on;
plot(tt5s,sum(fe_s,2),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
title('Total Extensor Muscle Force');
set(gca,'XTickLabel',{' '});
ylim([0 4500]);
set(gca,'YTickLabel',{' '});

subplot(2,3,3)
plot(tt5,sum(fe_c,2)+sum(ff_c,2),'r:','LineWidth',3);
hold on;
plot(tt5s,sum(fe_s,2)+sum(ff_s,2),'b-','LineWidth',3);
set(gca,'box','off','FontSize',12);
title('Total Muscle Force');
set(gca,'XTickLabel',{' '});
ylim([0 4500]);
set(gca,'YTickLabel',{' '});
legend('Cosimulation','Serial');

saveas(gcf,'./Serial_vs_Cosim.fig');
saveas(gcf,'./Serial_vs_Cosim.jpg');